function increaseSurfaces(aImarisApplicationID)

% Produce inner surface from clipped surfaces

%    <CustomTools>
%      <Menu>
%       <Submenu name="Surfaces Functions">
%        <Item name="Increase Surfaces" icon="Matlab" tooltip="Increases Surfaces Volume">
%          <Command>MatlabXT::increaseSurfaces(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%    </CustomTools>
%
% Copyright (c) 2018, Dominic Filion

% Description : The fonction allow to produce a surface increased by a
% ratio of units (or even decrease a surface if the input number is
% negative).



% Set up connection between Imaris and MATLAB
if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

% Get the Surpass scene
vSurpassScene = vImarisApplication.GetSurpassScene;

vSurface = [];

incSize = inputdlg({'Enter increasing size (in units):'},'Input',1,{'2'});
if isempty(incSize)
    return
end
incSize = str2double(incSize{1});

for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
    vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
    if vImarisApplication.GetFactory.IsSurfaces(vDataItem)
        vSurface = vImarisApplication.GetFactory.ToSurfaces(vDataItem);
        if vSurface.GetVisible  % Process only if the surface is selected
            % Run the algorithm for all different surfaces
            vIds = vSurface.GetIds();
            vSurfaceGroup = vImarisApplication.GetFactory.CreateDataContainer;
            name = char(vSurface.GetName());
            
            sprintf('Dilate %s',name)
            vSurfaceGroup.SetName(sprintf('Dilated from %s',name));
            for aSurfaceIndex = 0:size(vIds,1)-1
                
                % Process the surface if it is the current Time point
                if vSurface.GetTimeIndex(aSurfaceIndex) == vImarisApplication.GetVisibleIndexT
                    
                    % Get the surfaces components
                    vVertices = vSurface.GetVertices(aSurfaceIndex);
                    vTriangles = vSurface.GetTriangles(aSurfaceIndex);
                    vNormals = vSurface.GetNormals(aSurfaceIndex);
                    % Increase surface volume
                    vVertices = vVertices + incSize*vNormals;
                    newSurfaces = vImarisApplication.GetFactory.CreateSurfaces;
                    newSurfaces.AddSurface(vVertices, vTriangles, vNormals, 0);
                    vRGBA = vSurface.GetColorRGBA;
                    [R,G,B] = rgba2rgb(vRGBA);
                    newSurfaces.SetColorRGBA(rgb2rgba(R,G,B));
                    vSurfaceGroup.AddChild(newSurfaces, -1);
                end
            end
            vSurpassScene.AddChild(vSurfaceGroup, -1);
        end
    end
end

% Return the error that No Surfaces are selected
if isempty(vSurface)
    errordlg('Please Create a surface or select one!');
    return
end


%--------------------------------------------------------------------------
    function [R,G,B,A] = rgba2rgb(rgba)
        
        if rgba < 0
            rgba = rgba + 2^32;
        end
        str = dec2bin(rgba);
        strz = sprintf('%032d',0);
        strz(end-length(str)+1:end) = str;
        A = bin2dec(strz(1:8));
        B = bin2dec(strz(9:16));
        G = bin2dec(strz(17:24));
        R = bin2dec(strz(25:32));
    end
%--------------------------------------------------------------------------
    function rgba = rgb2rgba(R,G,B)
        
        rgba = bin2dec(sprintf('%08d%08d%08d',str2double(dec2bin(B)),...
            str2double(dec2bin(G)),str2double(dec2bin(R))));
    end

end
